*  Program...........: MAKEHELP.PRG
*  Author............: James P. Fisher Jr.
*  Project...........: MakeHelp.app
*  Created...........: 12/08/94 at 23:34
*  Copyright.........: (c) 1994 James P. Fisher Jr.
*) Description.......: This the heart of Makehelp.app.
*)                   : All of the actual processing takes place in this
*)                   : program.
*)                   :
*)                   :
*)                   :
*)                   :
*)                   :
*  Calling Samples...:
*  Parameter List....: lcHelpfile (the Help table), lcProject (the project)
*  Major change list.: v3.0 supports both Genscrnx and Genmenux
*) Dependencies......:
*)                   :
*)                   :

PARAMETERS lcHelpFile, lcProject
PRIVATE lcTitle, ;
   lcWName, ;
   lcHelpFile, ;
   lcProject, ;
   lcMenuName, ;
   lcScrName, ;
   lcOldHelp, ;
   lcOldHelpS, ;
   lcComment, ;
   lcTopic
m.lcTitle = ""
m.lcWName = ""
m.lcComment = ""
m.lcTopic = ""

*-- Select an unused work area and open the help table
SELECT 0
USE (lcHelpFile) ALIAS APPHELP EXCLUSIVE

*-- Clear out the old information ...
ZAP

*-- Select an unused work area and open the project table
SELECT 0
USE (lcProject) ALIAS PROJECT AGAIN

*-- Find the menu file(s) ...
SCAN FOR TYPE = "M"
   lcMenuName = PROJECT.NAME

   *-- Select an unused work area and open the menu table
   SELECT 0
   USE (lcMenuName) ALIAS MENU AGAIN

   *-- Let the user know what's happening
   WAIT WINDOW  "Generating menu level help for " + m.lcMenuName NOWAIT
   SCAN

      *-- If the record is not just a name ...
      IF Itemnum # "0"

         *-- and not a separator bar ...
         IF PROMPT # "\-"

            *-- If we have text to place in the help file ...
            IF !EMPTY(Menu.Comment)

               *-- Select the help table and scatter blank memvars ...
               SELECT APPHELP
               SCATTER MEMVAR MEMO BLANK

               *-- If we have defined a pad name ...
               IF !EMPTY(Menu.Name)

                  *-- Store '' + the properly stripped menu name to the topic field
                  *-- The test for '_' handles system bar names ...
                  m.Topic = " "+ IIF(LEFT(Menu.Name,1) <> ;
                     "_",PROPER(Menu.Name),PROPER(STRPPRMT(MENU.PROMPT)))

               ELSE

                  *-- We haven't defined a pad name so we'll strip the prompt of '\<'
                  *-- and use that ...
                  m.Topic = " "+PROPER(STRPPRMT(Menu.Prompt))

               ENDIF ( !EMPTY(Menu.Name) )

               *-- If we're using Genmenux we need to get the help text from between the
               *-- #STARTHELP and #ENDHELP help generator directives in the comment field
               *-- and add them to the details field with the topic as a header ...
               IF m.llGenmenux
                  m.Details = ;
                     "Ŀ" + CHR(13) + ;
                     ""+PADC(UPPER(ALLTRIM(SUBSTR(m.Topic,3))),33) + ;
                     "" + CHR(13) + ;
                     "" ;
                     + CHR(13) + CHR(13) + GETHELP(MENU.COMMENT)

               ELSE

                  *-- we're not using Genmenux
                  m.Details = ;
                     "Ŀ" + CHR(13) + ;
                     ""+PADC(UPPER(ALLTRIM(SUBSTR(m.Topic,3))),33) + ;
                     "" + CHR(13) + ;
                     "" ;
                     + CHR(13) + CHR(13) + MENU.COMMENT
               ENDIF ( m.llGenmenux )

               *-- Store the Levelname to the class field.
               *-- We'll use this later when cross referencing
               *-- the menu selections
               m.Class = Menu.LevelName
               APPEND BLANK
               GATHER MEMVAR MEMO
               SELECT MENU
            ENDIF ( !EMPTY(Menu.Comment) )
         ENDIF ( Prompt # "\-" )
      ENDIF ( Itemnum # "0" )
   ENDSCAN
   *-- Close the menu table ....
   USE
ENDSCAN ( FOR type = "M" )

*-- Go back to the project table ....
SELECT PROJECT

*-- Find the screen tables ....
SCAN FOR TYPE = "s"

   *-- We need to initialize some blank variables for each .scx
   m.lcTitle = ""
   m.lcWName = ""

   *-- Store the screen name to a variable and use it in a Wait Window
   *-- to tell the user what's happening
   m.lcScrName = PROJECT.NAME
   WAIT WINDOW "Generating window level help for " ;
      + STRIPPAT(m.lcScrName) NOWAIT

   *-- Select an unused work area and open the screen table ....
   SELECT 0
   USE (lcScrName) ALIAS SCREEN AGAIN

   *-- Limit our search for help text to objects that can make use of help
   *-- and limit the search to the selected platform
   SCAN FOR INLIST(Objtype,1,11,12,13,14,15,16,20) .AND. ;
         PLATFORM = m.lcPlatform

      *-- If the record is the screen header ....
      IF objtype = 1

         *-- If the window has a name ....
         *-- We need a window name to make the help context sensitive
         IF !EMPTY(SCREEN.NAME)

            *-- We need to find out what kind of screen table
            *-- we're dealing with
            DO CASE

                  *-- If the window is a modal or modalless read window
               CASE UPPER(LEFT(SCREEN.NAME,2)) ;
                     = "WR" .OR. UPPER(LEFT(SCREEN.NAME,2)) = "WM"

                  *-- Store the window name to our variable for processing
                  STORE SCREEN.NAME TO m.lcWName

                  *-- Process the variable
                  m.lcWName = ALLTRIM(m.lcWName)
                  m.lcWName = LOWER(m.lcWName)

                  *-- Store the window title to our variable for processing ....
                  m.lcTitle = SCREEN.TAG

                  *-- Process the window title, stripping the quotation
                  *-- characters
                  IF INLIST(LEFT(m.lcTitle,1),["],['],[(],"[")
                     m.lcTitle = SUBSTR(m.lcTitle,2,LEN(m.lcTitle)-2)
                  ENDIF
                  m.lcTitle = ALLTRIM(m.lcTitle)

                  *-- If the window is our toolbar ...
               CASE UPPER(LEFT(SCREEN.NAME,2)) = ;
                     "WC"

                  *-- Store the window name to our variable ....
                  STORE LOWER(NAME) TO lcWName

                  *-- If the window hasn't been given a name, we'll name it ....
                  IF EMPTY(Screen.Tag)
                     m.lcTitle = "Control Panel"
                  ELSE
                     *-- we'll use the existing name ....
                     m.lcTitle = SCREEN.TAG
                     IF INLIST(LEFT(m.lcTitle,1),["],['],[(],"[")
                        m.lcTitle = SUBSTR(m.lcTitle,2,LEN(m.lcTitle)-2)
                     ENDIF		 && Strip the quotation marks
                     m.lcTitle = ALLTRIM(m.lcTitle)
                  ENDIF
            ENDCASE
         ENDIF
      ELSE

         *-- The record isn't the screen header ...
         IF !EMPTY(SCREEN.NAME) 			 &&  If the window has a name

            *-- If we have some text to work with ....
            IF !EMPTY(MESSAGE) .OR. !EMPTY(COMMENT)
               m.lcMessage = Screen.Message
               m.lcMessage = ALLTRIM(m.lcMessage)
               m.lnMessLen = LEN(TRIM(m.lcMessage))-2
               m.lcMessage = SUBSTR(m.lcMessage,2,m.lnMessLen)	  &&  Strip the " 's
               m.lcComment = SCREEN.COMMENT                      &&  Store the memo to a string variable

               *-- If we have Genscrnx loaded we'll get the help text that lies between
               *-- the help generator directives #STARTHELP and #ENDHELP ....
               IF m.llGenscrnx
                  m.lcComment = GETHELP(m.lcComment)

               ELSE
                  *-- Genscrnx isn't loaded ...
                  *-- Note: We can tell the help generator that Genscrnx and/or Genmenux have
                  *-- been loaded and force it to use the generator directives.

                  m.lcComment = STRPPRMT(m.lcComment)
               ENDIF ( m.llGenscrnx )
               SELECT APPHELP
               SCATTER MEMVAR MEMO BLANK

               *-- We need to know what objects we are dealing with ...
               DO CASE

                     *-- If the object is a push button or check box ....
                  CASE INLIST(SCREEN.ObjType,12,14)

                     *-- We'll check for an assigned text object and store it to the topic ...
                     *-- If a text object hasn't been assigned we'll parse the Picture field to
                     *-- get a topic ...
                     m.lcTopic = IIF(EMPTY(TXTFIND(LOWER(SCREEN.Name))),PROPER(STRPPRMT(SCREEN.Picture)),TXTFIND(LOWER(SCREEN.Name)))
                     m.Topic = ALLTRIM(m.lcTopic)
                  OTHERWISE

                     *-- We'll check for an assigned text object and store it to the topic ...
                     *-- If a text object hasn't been assigned we'll use the window title ...
                     m.Topic = IIF(EMPTY(TXTFIND(LOWER(SCREEN.Name))),PROPER(m.lcTitle),TXTFIND(LOWER(SCREEN.Name)))
                     IF INLIST(LEFT(m.Topic,1),"^","~")
                        m.Topic = PROPER(SUBSTR(m.Topic,4))
                     ENDIF
               ENDCASE

               *-- We'll store the message text and concatenate any help text from
               *-- the comments field, using the topic as our header ....


               *-- Change Jim Fisher 10/08/94
               *-- draw a box around the header
               *               m.Details = PADC(ALLTRIM(m.Topic),37) + CHR(13) + CHR(13) + ;
               *                  ALLTRIM(m.lcMessage)+SPACE(2)+ m.lcComment

               m.Details = ;
                  "Ŀ" + CHR(13) + ;
                  ""+PADC(ALLTRIM(m.Topic),33) + ;
                  "" + CHR(13) + ;
                  "" ;
                  + CHR(13) + CHR(13) + ;
                  ALLTRIM(m.lcMessage)+SPACE(2)+ m.lcComment

               *-- end change 10/08/94


               *-- We'll need to store the get name to the Read_Item field since we'll be
               *-- using FoxPro's VARREAD() function in our SET TOPIC statement ....
               *-- We'll also need the window title and window name in the record ...
               m.Read_Item = IIF(LOWER(LEFT(SCREEN.Name,2)) = ;
                  "m.", SUBSTR(LOWER(SCREEN.Name),3), LOWER(SCREEN.Name))
               m.WTitle = m.lcTitle
               m.WName = m.lcWName
               APPEND BLANK
               GATHER MEMVAR MEMO
            ENDIF ( !EMPTY(MESSAGE) .OR. !EMPTY(COMMENT) )
         ENDIF ( !EMPTY(SCREEN.NAME) )
      ENDIF ( objtype = 1 )
   ENDSCAN
   USE
ENDSCAN

*-- Since the comments field is used to store a number of different
*-- directives, we may well have some empty records.
*-- We'll get rid of them now ....
SELECT APPHELP
GO TOP
WAIT WINDOW NOWAIT "Deleting empty records ..."

*-- An empty topic can't be found ...
SCAN FOR EMPTY(TOPIC)
   DELETE
ENDSCAN

*-- An empty memo field won't help anyone, nor will one that only contains
*-- the Topic ...
SCAN FOR EMPTY(DETAILS) .OR. EMPTY(READ_ITEM) .AND. !EMPTY(WNAME) .OR. ;
      EMPTY(WTITLE) .AND. EMPTY(wName) .AND. !EMPTY(Read_Item)
   DELETE
ENDSCAN

*-- This code handles MCONTROL.SCX which may occur numerous times in a
*-- Codebook application
INDEX ON WNAME TO TMPTOP.IDX
SET INDEX TO TMPTOP.IDX
GO TOP
IF SEEK("wcmcontrol")
   WAIT WINDOW "Deleting duplicate records ..." NOWAIT
   DO WHILE WNAME = "wcmcontrol"
      lcRead_Item = Read_Item
      lnOldRec = RECNO()
      SCAN FOR WNAME = "wcmcontrol"
         IF RECNO() <> lnOldRec .AND. Read_Item = lcRead_Item
            DELETE
         ENDIF
      ENDSCAN
      GO lnOldRec
      SKIP
   ENDDO
ENDIF
SET INDEX TO
DELETE FILE TMPTOP.IDX                           && Clean up ...

*-- I don't like to pack in an end user application but here, I really need to
PACK

*-- If our window name is empty, we'll assume we're dealing with one of the
*-- menu records ....
*-- We'll find the submenu topics and cross reference them ....
SCAN FOR EMPTY(wName)
   lcString = CHR(13)+CHR(13)+"See Also:  "
   lnRecord = RECNO()
   lcTopic = SUBSTR(TOPIC,3)
   *-- strip the spaces from the lcTopic since our class won't have
   *-- any spaces ( same as Menu.levelname )
   DO WHILE AT(' ',lcTopic) > 0
      lcTopic = SUBSTR(lcTopic,1,AT(' ',lcTopic)-1)+ ;
         ALLTRIM(SUBSTR(lcTopic,AT(' ',lcTopic)+1))
   ENDDO
   lcTopic = PADR(lcTopic,10)
   lcClass = CLASS
   SKIP

   *-- Let the user know what is taking place ....
   DO CASE
      CASE _DOS
         WAIT WINDOW NOWAIT ;
            "Cross Referencing "+ m.lcTopic
      CASE _WINDOWS
         WAIT WINDOW NOWAIT "Cross Referencing "+ ANSITOOEM(m.lcTopic)
   ENDCASE

   *-- This is where we use the class field
   *-- We'll build the cross reference text ....
   SCAN FOR UPPER(CLASS) = UPPER(lcTopic)
      IF EMPTY(wName)
         lcString = lcString+RTRIM(SUBSTR(TOPIC,3))+", "
      ENDIF
   ENDSCAN
   GO lnRecord
   IF CHR(13)+CHR(13)+"See Also:  " # (lcString)
      lcString = ALLTRIM(lcString)
      lcString = LEFT(lcString,LEN(lcString)-1)	 && Strip the last comma ...
      REPLACE Details WITH Details+lcString
   ENDIF
ENDSCAN
CLOSE DATABASES

*-- Let the user know that we're finished ....
WAIT WINDOW NOWAIT "Your help file is complete ....."

*-- Allow the user to examine the newly built help table ....
SET HELP ON
SET HELP TO &lcHelpFile
SET TOPIC TO
HELP
RETURN


*******************************************************************************
FUNCTION STRPPRMT       &&  Strips out junk that our users won't understand ...
*******************************************************************************


PARAMETERS lcFldName
PRIVATE lcFldName

IF INLIST(LEFT(m.lcFldName,1),["],['],[(],"[")
   *	m.lcFldName = SUBSTR(m.lcFldName,2,LEN(m.lcFldName)-2)
   m.lcFldName = CHRTRAN(m.lcFldName,'"()[]','    ')
   m.lcFldName = CHRTRAN(m.lcFldName,"'"," ")
ENDIF												 && Strip the quotation marks ...

m.lcFldName = ALLTRIM(m.lcFldName)

DO CASE

      *-- These are Codebook specific buttons used on the toolbar
      *-- we'll return the prompt names ....
   CASE UPPER("@*bhn ..\bmps\first1.bmp") $ m.lcFldName	&& MControl _WINDOWS
      m.lcFldName = "First"

   CASE UPPER("@*bhn ..\bmps\prev1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "Prior"

   CASE UPPER("@*bhn ..\bmps\next1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "Next"

   CASE UPPER("@*bhn ..\bmps\last1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "Last"

   CASE UPPER("@*bhn ..\bmps\list1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "List"

   CASE UPPER("@*bhn ..\bmps\find1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "Find"

   CASE UPPER("@*bhn ..\bmps\new1.bmp")  $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "New"

   CASE UPPER("@*bhn ..\bmps\edit1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "Edit"

   CASE UPPER("@*bhn ..\bmps\save1.bmp") $ m.lcFldName		&& MControl _WINDOWS
      m.lcFldName = "Save"

   CASE UPPER("@*bhn ..\bmps\cancel1.bmp") $ m.lcFldName	&& MControl _WINDOWS
      m.lcFldName = "Cancel"

   CASE UPPER("@*bhn ..\bmps\close1.bmp")  $ m.lcFldName	&& MControl _WINDOWS
      m.lcFldName = "Close"

      *-- end Codebook specific buttons used on the toolbar

   CASE UPPER("@*bh") $ m.lcFldName					&& Generic Horizontal Picture Button
      m.lcFldName = SUBSTR(m.lcFldName,RAT("\",m.lcFldName)+1,RAT(".",m.lcFldName)-RAT("\",m.lcFldName)-1)

   CASE UPPER("@*bv") $ m.lcFldName					&& Generic Vertical Picture Button
      m.lcFldName = SUBSTR(m.lcFldName,RAT("\",m.lcFldName)+1,RAT(".",m.lcFldName)-RAT("\",m.lcFldName)-1)
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*C " $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*C ",m.lcFldName)+4)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

      *-- This CASE statement must occur before "@*HN " since "@*HN " is a
      *-- substring of "@*HN \?" ...

   CASE "@*HN \?" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HN \?",m.lcFldName)+7)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*HN " $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HN ",m.lcFldName)+5)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

      *-- These are Codebook specific prompts

   CASE "@*HT \!OK;\?CANCEL" $ UPPER(m.lcFldName)
      m.lcFldName = " Ok   < Cancel >"

   CASE "@*HT \<MORE;\!\?\<DONE" $ UPPER(m.lcFldName)
      m.lcFldName = " More   < Done >"

      *-- end Codebook specific prompts

   CASE "@*HT \?\<" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HT \?\<",m.lcFldName)+9)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*HT \!\<" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HT \!\<",m.lcFldName)+9)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*HT \!\?" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HT \!\?",m.lcFldName)+9)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*HT \?" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HT \?",m.lcFldName)+7)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*HT \!" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HT \!",m.lcFldName)+7)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*HT " $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*HT ",m.lcFldName)+5)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*VT \!\<OK;\?\<CANCEL" $ UPPER(m.lcFldName)
      m.lcFldName = " Ok   < Cancel >"

   CASE "@*VT \?" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*VT \?",m.lcFldName)+7)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*VT \!" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*VT \!",m.lcFldName)+7)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*VT" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*VT",m.lcFldName)+4)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*VN \!COMPRESS;\?CANCEL" $ UPPER(m.lcFldName)
      m.lcFldName = " Compress   < Cancel >"

   CASE "@*VN \!INDEX;\?CANCEL" $ UPPER(m.lcFldName)
      m.lcFldName = " Index   < Cancel >"

   CASE "@*VN \!\<OK;\?\<CANCEL" $ UPPER(m.lcFldName)
      m.lcFldName = " Ok   < Cancel >"

   CASE "@*VN \!" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*VN \!",m.lcFldName)+7)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "@*VN " $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,ATC("@*VN ",m.lcFldName)+5)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "\<" $ m.lcFldName
      m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
         SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      DO WHILE "\<" $ m.lcFldName
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("\<",m.lcFldName)-1) + ;
            SUBSTR(m.lcFldName,ATC("\<",m.lcFldName)+2)
      ENDDO
      m.lcFldName = STRIPJNK(m.lcFldName)

   CASE "#DOSOBJ" $ UPPER(m.lcFldName) .OR. "#MACOBJ" $ UPPER(m.lcFldName) ;
         .OR. "#WINOBJ" $ UPPER(m.lcFldName) .OR. "#UNIXOBJ" $ UPPER(m.lcFldName) ;
         .OR. "#STARTHELP" $ UPPER(m.lcFldName) .OR. "#ENDHELP" $ UPPER(m.lcFldName)
      IF ATC("#DOSOBJ",m.lcFldName) > 0
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("#DOSOBJ",m.lcFldName)-1) + SUBSTR(m.lcFldName,ATC("#DOSOBJ",m.lcFldName)+7)
      ENDIF
      IF ATC("#MACOBJ",m.lcFldName) > 0
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("#MACOBJ",m.lcFldName)-1) + SUBSTR(m.lcFldName,ATC("#MACOBJ",m.lcFldName)+7)
      ENDIF
      IF ATC("#WINOBJ",m.lcFldName) > 0
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("#WINOBJ",m.lcFldName)-1) + SUBSTR(m.lcFldName,ATC("#WINOBJ",m.lcFldName)+7)
      ENDIF
      IF ATC("#UNIXOBJ",m.lcFldName) > 0
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("#UNIXOBJ",m.lcFldName)-1) + SUBSTR(m.lcFldName,ATC("#UNIXOBJ",m.lcFldName)+8)
      ENDIF
      IF ATC("#STARTHELP",m.lcFldName) > 0
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("#STARTHELP",m.lcFldName)-1) + SUBSTR(m.lcFldName,ATC("#STARTHELP",m.lcFldName)+10)
      ENDIF
      IF ATC("#ENDHELP",m.lcFldName) > 0
         m.lcFldName = SUBSTR(m.lcFldName,1,ATC("#ENDHELP",m.lcFldName)-1) + SUBSTR(m.lcFldName,ATC("#ENDHELP",m.lcFldName)+8)
      ENDIF
      *	OTHERWISE                                  &&  Debugging code
      *		m.lcFldName = "Add case statement ..."  &&  Debugging code

ENDCASE


RETURN m.lcFldName

*******************************************************************************
FUNCTION TXTFIND    && Finds the proper text object to go with a get object ...
*******************************************************************************
PARAMETERS lcGet
PRIVATE lnOldRec, RetVal
SELECT SCREEN
m.lnOldRec = RECNO()
LOCATE FOR SCREEN.ObjType = 5 .AND. LOWER(m.lcGet) $ LOWER(SCREEN.Comment)
*               && Find the text object containing the get in its Comment field
IF FOUND()  && If we remembered to add the get name to the text object 'Comment ...'
   m.RetVal = SCREEN.Expr


   *-- Change Jim Fisher 10/08/94
   *-- use Stripjnk() to get rid of the carriage returns and linefeeds


   m.RetVal = Stripjnk(m.RetVal)

   *-- end change


   IF INLIST(LEFT(m.RetVal,1),["],['],[(],"[")
      m.RetVal = SUBSTR(m.RetVal,2,LEN(m.RetVal)-2)    &&  Strip the quotation marks
   ENDIF

   m.RetVal = ALLTRIM(m.RetVal)


   *-- Change Jim Fisher 10/08/94
   *-- we need to strip out strings of spaces and quotes



   DO WHILE "  " $ m.RetVal
      m.RetVal = STRTRAN(m.RetVal,"  "," ")
   ENDDO

   *-- end change


ELSE
   m.RetVal = ""                                 &&  We'll return an empty text string
ENDIF
GO m.lnOldRec
SELECT APPHELP
RETURN m.RetVal



*******************************************************************************
FUNCTION GETHELP
*******************************************************************************
PARAMETERS lcHelp
PRIVATE lcHelp, lnStart, lnEnd, lcRetHelp
IF "#STARTHELP" $ UPPER(m.lcHelp) ;
      .AND. "#ENDHELP" $ UPPER(m.lcHelp)                     && If we have help directives
   m.lnStart = ATC("#STARTHELP",m.lcHelp) + 10               && Find out where the help text starts
   m.lnEnd = ATC("#ENDHELP",m.lcHelp) - m.lnStart            && Find out where the help text ends
   m.lcRetHelp = ALLTRIM(SUBSTR(m.lcHelp,m.lnStart,m.lnEnd)) && Pull the help text from the comment
ELSE
   m.lcRetHelp = ""                                          && There's nothing to do
ENDIF
RETURN m.lcRetHelp                                           && Return the processed comment



******************************************************************************
PROCEDURE STRIPJNK
******************************************************************************
PARAMETERS jcFldName
m.jcFldName = STRTRAN(m.jcFldName,";"," ")
m.jcFldName = STRTRAN(m.jcFldName,"\<","")
m.jcFldName = STRTRAN(m.jcFldName,"\!","")
m.jcFldName = STRTRAN(m.jcFldName,"\?","")
m.jcFldName = STRTRAN(m.jcFldName,CHR(10),CHR(0))
m.jcFldName = STRTRAN(m.jcFldName,CHR(13),CHR(0))
RETURN m.jcFldName
